/*
 * Copyright (c) 2002 Isao SEKI <iseki@gongon.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is included in WiStunbler package.
 *
 * $Id: stumbler.c,v 1.4 2002/08/11 14:18:51 iseki Exp $
 * $Id: stumbler.c,v 1.5 2003/11/14 15:54:29 pancake Exp $
 */

#include <sys/types.h>
#include <sys/cdefs.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h> 
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <stdio.h>
#include <string.h> 
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h> 
#include <errno.h>
#include <err.h>
#include <time.h>
#include <signal.h>
#if USE_GTK
#include <gtk/gtk.h>
#include "guimenu.h"
#endif
#include "stumbler.h"
#include "gpscontrol.h"
#include "speaker.h"

extern void wistumbler_sniff();
struct access_point *access_points[MAX_APS];
#if USE_GTK
struct _app app;
gint timer;
gchar *titles[12] = {"", "SSID", "BSSID", "Channel", "Signal", "Interval",
                     "Capinfo", "First", "Last", "Rate", "Position"};
#endif

int gps = 0;
int gtk=0;
int debug = 0;
int usex = 1;
char *logsfile = NULL;
char *iface1 = NULL;
char *gpsdev = NULL;
int scan_delay=2;
int working=1;

void window_setup()
{
#if USE_GTK
    GtkWidget *vbox, *hbbox, *quit_button, *save_button, *scrolled_win;
    GtkWidget *wifi_label,*hbox;

    if (!usex) return;

    app.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_position(GTK_WINDOW(app.window), GTK_WIN_POS_CENTER);
    gtk_window_set_title(GTK_WINDOW (app.window), "wistumbler2");
    gtk_signal_connect(GTK_OBJECT(app.window), "destroy",
                       GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

    vbox = gtk_vbox_new(FALSE, 5);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
    gtk_container_add(GTK_CONTAINER(app.window), vbox);

    /* create MenuBar */
    wistumbler_gui_menu(app.window,vbox);

    /* create scrolled window */
    scrolled_win = gtk_scrolled_window_new(NULL, NULL);
    gtk_container_set_border_width(GTK_CONTAINER(scrolled_win), 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize(GTK_WIDGET(scrolled_win), 700, 100);
    gtk_box_pack_start(GTK_BOX (vbox), scrolled_win, TRUE, TRUE, 0);

    /* create GtkCList */
    app.clist = gtk_clist_new_with_titles(11, titles);
    gtk_clist_column_titles_passive((GtkCList *)app.clist);
/*    gtk_clist_set_column_justification((GtkCList *)app.clist, 0,
                                                         GTK_JUSTIFY_LEFT); */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 0, 10);  /* receive */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 1, 200); /* name */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 2, 200); /* BSSID */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 3, 80);  /* channel */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 4, 60);  /* signal */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 5, 60);  /* interval */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 7, 60);  /* First */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 8, 60);  /* last */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 9, 35);  /* rate */
    gtk_clist_set_column_width (GTK_CLIST(app.clist), 10, 100);  /* position */
    gtk_container_add(GTK_CONTAINER(scrolled_win), app.clist);

    /* create Save &  Quit button */
    hbbox = gtk_hbutton_box_new();
    	gtk_box_pack_start(GTK_BOX(vbox), hbbox, FALSE, FALSE, 5);
    	gtk_container_set_border_width(GTK_CONTAINER(hbbox), 0);
    	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbbox), GTK_BUTTONBOX_END);
    	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbbox), 5);

    /* Horitzontal Box for !button widgets */
    hbox = gtk_hbutton_box_new();
	gtk_widget_show(hbox);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_START);
    	gtk_container_add(GTK_CONTAINER(hbbox),hbox);

    /* WIFI LABEL */
    wifi_label = gtk_label_new(iface1);
	gtk_container_add(GTK_CONTAINER(hbox), wifi_label);
    /* SAVE BUTTON */
    save_button = gtk_button_new_from_stock(GTK_STOCK_SAVE);
    	gtk_signal_connect_object(GTK_OBJECT(save_button), "clicked",
            GTK_SIGNAL_FUNC(print_clist),(gpointer)1);
	gtk_container_add(GTK_CONTAINER(hbbox), save_button);
    /* QUIT BUTTON */
    quit_button = gtk_button_new_from_stock(GTK_STOCK_QUIT);
    	gtk_signal_connect_object(GTK_OBJECT(quit_button), "clicked",
            GTK_SIGNAL_FUNC(wi_exit), (gpointer)0);
	gtk_container_add(GTK_CONTAINER(hbbox), quit_button);

    /* Show All! */
    gtk_widget_show_all(app.window); 
#endif
}

void print_clist(save)
int save;
{
    int i, j;
    float rate;
    struct tm *tm;
    struct access_point *w;
    char receive[2], name[33], bssid[18], channel[3], signal[4], interval[6],
          capinfo[4], first[6], last[6], speed[4], position[32];
    char *line[12] = {receive, name, bssid, channel, signal, interval, capinfo,
                       first, last, speed, position};
    int maxsignal=0; /* for beep purposes */
    struct time_t *tmt;
    FILE *fd=NULL;

#if USE_GTK
    if (usex)
        gtk_clist_clear((GtkCList *)app.clist);
    else {
#endif
	fd = fopen ("/dev/stdout","w");
	if (!fd) {
		printf("Error: Cannot open console for output.\n");
		exit(1);
	}
	fprintf(fd,"\e[2J"); /* clear screen */
	fflush(fd);
#if USE_GTK
    }
#endif

    if (save) {
	fd = fopen (logsfile,"a");
	if (!fd) {
		fprintf(stderr,"Cannot append logs to %s.\n",logsfile);
		return;
	}
	/* Print date of log snapshot */
	tmt=(struct time_t*)time(0);
	fprintf(fd,"--[ %s", ctime((const time_t*)&tmt)); /* \n */
    }

    for (i=0; i < MAX_APS; i++) {
        if (access_points[i] == NULL)
            continue;

        w = access_points[i];

        /* for 1st column, receive status */
        if (w->cur_receive == 0) {
            receive[0] = ' ', receive[1] = '\0';
        } else {
            receive[0] = 'o', receive[1] = '\0';
            w->cur_receive = 0;
        }

        for (j=0; j<w->wi_apinfo.namelen && j < sizeof name - 1; j++) {
            name[j] = w->wi_apinfo.name[j];
        }
        name[j] = '\0';
     
        sprintf(bssid, "%02x:%02x:%02x:%02x:%02x:%02x",
                      w->wi_apinfo.bssid[0]&0xff, w->wi_apinfo.bssid[1]&0xff,
                      w->wi_apinfo.bssid[2]&0xff, w->wi_apinfo.bssid[3]&0xff,
                      w->wi_apinfo.bssid[4]&0xff, w->wi_apinfo.bssid[5]&0xff);

        snprintf(channel, sizeof channel, "%d", w->wi_apinfo.channel);
        snprintf(signal, sizeof signal, "%d", w->wi_apinfo.signal);
        snprintf(interval, sizeof interval, "%d", w->wi_apinfo.interval);

	if (w->wi_apinfo.signal>maxsignal)
		maxsignal=w->wi_apinfo.signal;

	/* TODO : IBSS support? */
        if (w->wi_apinfo.capinfo & IEEE80211_CAPINFO_PRIVACY)
            sprintf(capinfo, "WEP");
        else if (w->wi_apinfo.capinfo & IEEE80211_CAPINFO_ESS)
            sprintf(capinfo, "ESS");
	else
	    capinfo[0] = '\0';
        
        tm = localtime((const time_t *)&w->first);
        sprintf(first, "%2.2d:%2.2d", tm->tm_hour, tm->tm_min);
        tm = localtime((const time_t *)&w->last);
        sprintf(last, "%2.2d:%2.2d", tm->tm_hour, tm->tm_min);

	rate=(int)w->wi_apinfo.rate/10;
#if 0
        switch (w->wi_apinfo.rate) {
            case WI_APRATE_1:
                rate = 1;
                break;
            case WI_APRATE_2:
                rate = 2;
                break;
            case WI_APRATE_5:
                rate = 5.5;
                break;
            case WI_APRATE_11:
                rate = 11;
                break;
            case WI_APRATE_0:
            default:
		printf("RATE VALUE: %d\n",w->wi_apinfo.rate);
                rate = 0;
                break;
        }
#endif
        if (rate)
            snprintf(speed, sizeof speed, "%g", rate);
        else
	    sprintf(speed,"--");
	    //speed[0] = '\0';

        if (gps)
            snprintf(position, sizeof position, "%f%c/%f%c",
                                w->dir.Ndeg, w->dir.N, w->dir.Edeg, w->dir.E);
	else position[0] = '\0';

	if (!usex||save) {
	/* TODO console GPS support */
	    fprintf(fd,"%s %s ch:%s sig:%s int:%s %s %s-%s spd:%s\n",
		name,bssid,channel,signal,interval,capinfo,first,last,speed);
	    fclose(fd);
	}
#if USE_GTK
	if (usex)
            gtk_clist_append((GtkCList *)app.clist, line);
#endif

    }
	/* Beep */
	if (!save)
	   do_beep(maxsignal);
}

int wi_polling()
{
    gps_read();

    if (app.init){
    if (iface1 == NULL )
	{
#if USE_GTK
	wistumbler_gui_error("No wifi devices found");
#else
	printf("No wifi devices found");
	if (!usex)
		working=0;
	//wi_exit(1);
#endif
	/* No wifi devices found */
	return 0;
	} else {

    if (wi_apscan(iface1) == -1) {
#if USE_GTK
	wistumbler_gui_error("This is not a valid interface");
#ifdef NOTHREADS
        gtk_timeout_remove(timer);
#endif
#else
	printf("Invalid iface\n");
	wi_exit(1);
#endif
    }

    if (debug)
        print_aps(stdout);

    print_clist(0);
    } }
    return(TRUE);
}

void th_polling()
{
	while (!app.init) usleep(100);
	if (!usex&&!iface1) /* nowork */
		return;
	while(working)
	{
	 wi_polling();
	 sleep(scan_delay);
	}
	pthread_exit(0);
}

int wi_exit(code)
int code;
{
    FILE *fp;

    printf("WiFinish..\n");
    wi_finish(iface1);		/* power down the card , if needed */
    gps_close();

    if (logsfile == NULL)
	exit(code);

    if ((fp = fopen(logsfile, "a")) == NULL) {
        perror("fopen");
        exit(1);
    } else {
        printf("Writing log ...\n\n");
        print_aps(fp);
        fclose(fp);
        exit(code);
    }
}

static void cleanup(dummy)
int dummy;
{
    wi_exit(0);
}

void
dev_init()
{
	/* Init devices (wifi/gps) */
    if (iface1 == NULL)
	iface1=autoget_wifi();
    if (iface1 == NULL)
	{
	printf("No wifi devices found.\n");
#ifndef USE_GTK
	usage(filename);
	wi_exit(1);
#endif
	}
    signal(SIGINT, cleanup);
    signal(SIGQUIT, cleanup);
    signal(SIGHUP, cleanup);
    signal(SIGTERM, cleanup);

    if (iface1)
	{
    	if (wi_init(iface1) < 0)  /* power on the card, if down */
    	{
		if(usex) wistumbler_gui_error("Error initializiing wifi");
		else printf("Error initilizing wifi\n");
		working=0;
    	}

    	if (gps)
       	   if (gps_open(gpsdev) <= 0)
		{
		if (usex) wistumbler_gui_error("Error initializing gpsdev");
		else  printf("GPS ERROR\n");
               //exit(1);
		}
   	app.init=1;
	}
}


void usage(char *progname)
{
        fprintf(stderr,
            "usage: %s"
            " [-f logsFile] [-g gpsDev ] [-d] [-x] [-b] [ interface ]"
	    "\n",progname);
        exit(1);
}

int main(argc, argv)
int argc;
char *argv[];
{
    int i, c;
    char *filename=
#if defined(__OpenBSD__)
    argv[0];
#else
    strdup(getprogname());
#endif
#ifndef NOTHREADS
    pthread_t th1,th2,th3; /* th_polling , dev_init, wistumbler_sniff */
#endif

    if (argc > 1 && argv[1][0] != '-') {
        iface1 = argv[1];
        optind++;
    }

    while ((c = getopt(argc, argv, "bxdf:g:hi:")) != -1) {
        switch (c) {
            case 'd':
                debug = 1;
                break;
	    case 'b':
		beep_enable = 0;
		break;
	    case 'x':
		usex = 0;
		break;
            case 'f':
                logsfile = optarg;
                break;
            case 'g':
                gps = 1;
                gpsdev = optarg;
                break;
	    case 'i':
		iface1 = optarg;
		break;
            case 'h':
            default:
                usage(filename);
        }
    }

    argc -= optind;
    argv += optind;

    /* Load configuration file */
    app.init=0; /* not yet init */
    user_conf();

	/* UGLY ORIGINAL CODE MUST BE PATCHED ( TODO ) */
    for (i = 0; i <= MAX_APS; i++) 
        access_points[i] = NULL;

#ifndef NOTHREADS
	//if (usex)
	pthread_create(&th1,NULL,(void*)&th_polling,(void *)0);
	pthread_create(&th2,NULL,(void*)&dev_init,(void *)0);
#else
	dev_init();
#endif

#if USE_GTK

     if (usex){
     if (gtk_init_check(&argc,&argv)==FALSE)
      {
		printf("Cannot run the Xwindow system...going to console\n");
		usex=0;
      } else {
        gtk_init(&argc, &argv);   /* init gtk   */
	gtk_set_locale();         /* init locales */
	gtk_rc_add_default_file("~/.gtkrc-2.0");
	gtk_rc_add_default_file("~/.gtkrc");
	gtk_rc_reparse_all();

        window_setup();

	#ifdef NOTHREADS
	timer = gtk_timeout_add(2000, (GtkFunction)wi_polling, NULL);
	#else
	#ifdef USE_PCAP
	pthread_create(&th3,NULL,(void*)wistumbler_sniff,(void*)0);
	#endif
	#endif
	gtk=1;
        gtk_main();        /* event loop of GTK */
    }}
#endif
	working=1; /* pthread_cancel could be faster :? TODO */
#ifndef NOTHREADS
	pthread_join(th1,NULL);
	printf("Joining..\n1\n");
	pthread_join(th2,NULL);
	printf("..2\n");
#if USE_PCAP
	pthread_join(th3,NULL);
	printf("..3\n");
#endif
	printf("Joined\n");
#endif
#if 0
    if (!usex)
	working=TRUE;
#ifdef NOTHREADS
	while(working) {
	    wi_polling();
	    sleep(scan_delay);
	}
#else
	    pthread_join(th1,NULL);
#if USE_PCAP
	    pthread_join(th2,NULL);
#endif
#if USE_GTK
    }
#endif
#endif
#endif

    wi_exit(0);
	printf("finish\n");

    return(0);
}
