/*
 * Copyright (c) 2002 pancake <pancake@phreaker.net>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is included in WiStumbler package.
 *
 * $Id: userconf.c,v 1.0 2003/11/29 15:54:29 pancake Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#define DEFAULT_LOGSFILE "wistumbler2.log"
#define DEFAULT_CFGFILE  ".wistumbler2.conf"

extern int scan_delay;
extern int beep_enable;
extern char *iface1;
extern char *logsfile;
extern int usex;

int
parsebool(char *str,int *bool)
{
	char *truestr[]={"yes","1","YES","on"};
	char *falsestr[]={"no","0","NO","off"};
	int i;

	for(i=0;i<4;i++)
		if (!strcmp(str,truestr[i]))
			{ *bool=1; break; }
	if (i==4)
	for(i=0;i<4;i++)
		if (!strcmp(str,falsestr[i]))
			{ *bool=0; break; }
	if (i==4)
		{
		fprintf(stderr,
		"Invalid boolean argument '%s'.\n",str);
		return 1;
		}
	return 0;
}

int
user_conf()
{
	FILE *fd;
	char buf[1024],*p;
	char one[1024],two[1024];
	char *home=getenv("HOME");
	if (home) chdir(home);

	fd=fopen(DEFAULT_CFGFILE,"r");
	if (!fd) {
		fd=fopen("/etc/wistumbler2.conf","r"); /* XXX use PREFIX */
	}

	if (fd)
	{
	while(!feof(fd))
	{
	one[0]=0;two[0]=0;buf[0]=0;
	fgets(buf,1024,fd);
	if ((p=strchr(buf,'=')))
		p[0]=' ';
	sscanf(buf,"%s %s",one,two);
	if (one[0]=='#') continue;
	if (one[0]&&two[0])
	   {
		if (!strcmp(one,"iface")) {
			if (iface1) free(iface1);
			if (!strcmp(two,"auto")) iface1=0;
			else iface1=strdup(two);
		} else if (!strcmp(one,"delay")) {
			scan_delay=atoi(two);
		} else if (!strcmp(one,"logsfile")) {
			if (logsfile) free(logsfile);
			logsfile=strdup(two);
		} else if (!strcmp(one,"console")) {
			if ( parsebool(two,&usex) )
				usex=1;
		} else if (!strcmp(one,"speaker")) {
			if ( parsebool(two,&beep_enable) )
				beep_enable=1;
		} else {
			fprintf(stderr,"Unknown directive: '%s'.\n",one);
			fclose(fd);
			return 1;
		}
	   }
	}
	fclose(fd);
	}
	if (!logsfile)
		logsfile=strdup(DEFAULT_LOGSFILE);

	return (0);
}
